unit DataObjects;

// $Id: DataObjects.pas,v 1.4 2002/02/25 17:56:56 takao Exp $

{$ObjExportAll On}

interface

uses
  ActiveX, Classes, Graphics, Windows;

type
  HGLOBAL = type LongInt;
  {$EXTERNALSYM HGLOBAL}

  TEnumFormatEtcProc = procedure(const aFormatEtc: TFormatEtc) of Object;

  { TDataObjectReader }
  TDataObjectReader = class
  private
    FDataObject: IDataObject;

    storageMedium_: TStgMedium;
    storageMediumValid_: Boolean;

  protected
    procedure ReleaseStorageMedium;

    function GetAsText: string;
    procedure SetDataObject(aDataObject: IDataObject);

  public
    constructor Create(aDataObject: IDataObject = Nil);
    destructor Destroy; override;

    procedure GetFormats(aProc: TEnumFormatEtcProc);
    function HasFormat(aFormat: Word): Boolean;

    function GetAsHandle(aFormat: Word; aTymed: Longint = TYMED_HGLOBAL): HGLOBAL;
    procedure GetAsFileList(files: TStrings);
    procedure GetAsBitmap(aBitmap: Graphics.TBitmap);

    property AsText: string read GetAsText;
    property DataObject: IDataObject read FDataObject write FDataObject;
  end;


implementation

uses
  ShellApi;

{ TDataObjectReader }

constructor TDataObjectReader.Create(aDataObject: IDataObject);
begin
  FDataObject := aDataObject;
end;

destructor TDataObjectReader.Destroy;
begin
  inherited;
end;

function TDataObjectReader.GetAsHandle(aFormat: Word;
  aTymed: Integer): HGLOBAL;
var
  formatEtc: TFormatEtc;
begin
  Result := 0;
  if FDataObject = Nil then Exit;

  formatEtc.cfFormat := aFormat;
  formatEtc.dwAspect := DVASPECT_CONTENT;
  formatEtc.lindex := -1;
  formatEtc.ptd := Nil;
  formatEtc.tymed := aTymed;

  ReleaseStorageMedium;

  if S_OK <> FDataObject.GetData(formatEtc, storageMedium_) then Exit;

  storageMediumValid_ := True;
  Result := storageMedium_.hGlobal;
end;

procedure TDataObjectReader.GetAsBitmap(aBitmap: Graphics.TBitmap);
var
  handle: THandle;
begin
  handle := GetAsHandle(CF_HDROP, TYMED_GDI);  // type of medium : gdi
  if handle = 0 then Exit
  else aBitmap.Handle := handle;
end;

procedure TDataObjectReader.GetAsFileList(files: TStrings);
var
  handle: THandle;
  i, n: Integer;
  buff: array[0..255] of Char;
begin
  handle := GetAsHandle(CF_HDROP);
  if handle = 0 then Exit;

  n := DragQueryFile(handle, $ffffffff, Nil, 0);
  for i := 0 to n - 1 do begin
    DragQueryFile(handle, i, buff, 255);
    files.Add(buff);
  end;
  DragFinish(handle);
end;

procedure TDataObjectReader.GetFormats(aProc: TEnumFormatEtcProc);
var
  enum: IEnumFORMATETC;
  i, count: Integer;
  formats: array[0..99] of TFormatEtc;
begin
  enum := Nil;
  if SUCCEEDED(FDataObject.EnumFormatEtc(DATADIR_GET, enum)) then begin
    count := 100;
    enum.Next(count, formats, @count);
    for i := 0 to count - 1 do aProc(formats[i]);
  end;
end;

function TDataObjectReader.GetAsText: string;
var
  handle: THandle;
  mem: Pointer;
  size: Integer;
  text: string;
begin
  Result := '';

  handle := GetAsHandle(CF_TEXT);
  if handle = 0 then Exit;

  size := GlobalSize(handle);
  mem := GlobalLock(handle);
  try
    SetLength(text, size);
    CopyMemory(PChar(text), mem, size);
  finally
    GlobalUnlock(handle);
  end;

  Result := text;
end;

function TDataObjectReader.HasFormat(aFormat: Word): Boolean;
var
  format: TFormatEtc;
begin
  format.cfFormat := aFormat;
  format.dwAspect := DVASPECT_CONTENT;
  format.lindex := -1;
  format.ptd := Nil;
  format.tymed := TYMED_HGLOBAL;

  Result := S_OK = FDataObject.QueryGetData(format);
end;

procedure TDataObjectReader.ReleaseStorageMedium;
begin
  if storageMediumValid_ then begin
    ReleaseStgMedium(storageMedium_);
    storageMediumValid_ := False;
  end;
end;

procedure TDataObjectReader.SetDataObject(aDataObject: IDataObject);
begin
  FDataobject := aDataObject;
end;

end.

